import argparse
import subprocess
import sys
import json
import time
import os
import curses

SyncerProc = None
SyncStrs = ['*------', '-*-----', '--*----', '---*---', '----*--', '-----*-', '------*']

DetailReportPath = '/tmp/RediSearchSyncerDetailReport_1.json'

MAX_SYNCERS_TO_SHOW = 30
StartSyncer = 0

parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter,
                                 description='Run RediSearch syncer')

parser.add_argument(
    '--source', '-s', default=None,
    help='source address')

parser.add_argument(
    '--destination', '-d', default=None,
    help='destination address')

parser.add_argument(
    '--is-light', '-l', action='store_const', const=True, default=False,
    help='Specify to sync from RediSearch light version')

parser.add_argument(
    '--add-prefix', '-p', default=None,
    help='prefix to add to all the hashes')

parser.add_argument(
    '--syncer-log-file', default='./syncer.log',
    help='Path to the syncer log file')

parser.add_argument(
    '--non-interactive', '-n', action='store_const', const=True, default=False,
    help='Run without interactive status shell')


args = parser.parse_args(args=sys.argv[1:])

f = open(args.syncer_log_file, 'wt')

stdoutPipe = f
stderrPipe = f
stdinPipe = subprocess.PIPE

options = {
            'stderr': stderrPipe,
            'stdin': stdinPipe,
            'stdout': stdoutPipe,
            'env': {'LD_LIBRARY_PATH' : './'}
          }

cmd = ['rl_syncer', '-d', args.destination, '-s', '1:0:%s' % args.source, '-F', '-b', '1', '-J']

if args.add_prefix is not None:
	cmd += ['-a', args.add_prefix] # add prefix

if not args.is_light:
	cmd += ['-T'] # remove tags from hashes

print(cmd)

SyncerProc = subprocess.Popen(cmd, **options)
try:
	os.remove(DetailReportPath)
except Exception:
	pass

def Report(stdscr):
	global StartSyncer
	while True:
		stdscr.clear()
		line = 0
		if os.path.exists(DetailReportPath):
			with open(DetailReportPath, 'rt') as f:
				data = json.loads(f.read())
				stdscr.addstr(line, 0, 'q - exits, F5 - refresh, UP|DOWN - navigate')
				line+=1
				stdscr.addstr(line, 0, 'status: %s' % data['mState'])
				line+=1
				source = data['sourceManagers'][0]
				stdscr.addstr(line, 0, '%s:' % source['Name'])
				line+=1
				if source['shardSyncers'] is not None:
					for i in range(StartSyncer, StartSyncer + MAX_SYNCERS_TO_SHOW, 1):
						if(i >= len(source['shardSyncers'])):
							break
						shards = source['shardSyncers'][i]
						stdscr.addstr(line, 8, '%d) %s:\tstatus:%s,\terrors:%s,\trepl_offset:%s' % (i + 1, shards['Name'], shards['mState'], str(shards['errors']), str(shards['dst_repl_offset'])))
						line+=1

					stdscr.refresh()
					while True:
		 				key = stdscr.getkey()
		 				if key == 'KEY_UP':
		 					if StartSyncer > 0:
		 						StartSyncer = StartSyncer - 1
		 					break
		 				if key == 'KEY_DOWN':
		 					if StartSyncer < len(source['shardSyncers']) - MAX_SYNCERS_TO_SHOW:
		 						StartSyncer = StartSyncer + 1
		 					break
		 				if key == 'KEY_F(5)':
		 					break
		 				if key == 'q':
		 					return
				else:
					stdscr.addstr(line, 8, 'Shard report are not yet available')
					line+=1
					stdscr.refresh()
					time.sleep(1)
					continue
		else:
			stdscr.addstr(0, 0, 'Waiting for report ...')
			stdscr.refresh()
			time.sleep(1)
			continue

if not args.non_interactive:
	curses.wrapper(Report)
else:
	while True:
		try:
			with open(DetailReportPath, 'rt') as f:
				print('-----------------------------------------------------------------')
				data = json.loads(f.read())
				print('status: %s' % data['mState'])
				source = data['sourceManagers'][0]
				print('%s:' % source['Name'])
				if source['shardSyncers'] is not None:
					for shards in source['shardSyncers']:
						print('\t%s:\tstatus:%s,\terrors:%s,\trepl_offset:%s' % (shards['Name'], shards['mState'], str(shards['errors']), str(shards['dst_repl_offset'])))
				print('-----------------------------------------------------------------')
				time.sleep(5)
		except Exception as e:
			pass


SyncerProc.kill()

SyncerProc.wait()

		

